@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+﻿#!/usr/bin/perl
 
 =head1 NAME
 
@@ -6,7 +6,7 @@ Asterisk::AMI - Perl moduling for interacting with the Asterisk Manager Interfac
 
 =head1 VERSION
 
-0.1.7
+0.1.8
 
 =head1 SYNOPSIS
 
@@ -30,6 +30,15 @@ reliable way to interact with Asterisk upon which other applications may be buil
 can integrate very easily into event-based applications, but it still provides blocking functions for us with standard
 scripting.
 
+=head2 SSL SUPPORT INFORMAION
+
+For SSL support you will also need the module that AnyEvent::Handle uses for SSL support, which is not a required dependency.
+Currently that module is 'Net::SSLeay' (AnyEvent:Handle version 5.251) but it may change in the future.
+
+=head3 CentOS/Redhat
+
+If the version of Net:SSLeay included in CentOS/Redhat does not work try installing an updated version from CPAN.
+
 =head2 Constructor
 
 =head3 new([ARGS])
@@ -42,6 +51,8 @@ Creates a new AMI object which takes the arguments as key-value pairs.
 	Events		Enable/Disable Events		'on'|'off'
 	Username	Username to access the AMI
 	Secret		Secret used to connect to AMI
+	AuthType	Authentication type to use for login	'plaintext'|'MD5'
+	UseSSL		Enables/Disables SSL for the connection	0|1
 	BufferSize	Maximum size of buffer, in number of actions
 	Timeout		Default timeout of all actions in seconds
 	Handlers	Hash reference of Handlers for events	{ 'EVENT' => \&somesub };
@@ -60,16 +71,19 @@ Creates a new AMI object which takes the arguments as key-value pairs.
 	login action.
 	'Username' has no default and must be supplied.
 	'Secret' has no default and must be supplied.
+	'AuthType' sets the authentication type to use for login. Default is 'plaintext'.  Use 'MD5' for MD5 challenge
+	authentication.
+	'UseSSL' defaults to 0 (no ssl). When SSL is enabled the default port changes to 5039.
 	'BufferSize' has a default of 30000. It also acts as our max actionid before we reset the counter.
 	'Timeout' has a default of 0, which means no timeout or blocking.
-	'Handlers' accepts a hash reference setting a callback handler for the specfied event. EVENT should match the what
+	'Handlers' accepts a hash reference setting a callback handler for the specified event. EVENT should match the what
 	the contents of the {'Event'} key of the event object will be. The handler should be a subroutine reference that
 	will be passed the a copy of the AMI object and the event object. The 'default' keyword can be used to set
-	a default event handler. If handlers are installed we do not buffer events and instead immediatly dispatch them.
+	a default event handler. If handlers are installed we do not buffer events and instead immediately dispatch them.
 	If no handler is specified for an event type and a 'default' was not set the event is discarded.
 	'Keepalive' only works when running with an event loop.
-	'TCP_Keepalive' default is disabled. Actives the tcp keepalive at the socket layer. This does not require 
-	an eventloop and is lightweight. Useful for applications that use long-lived connections to Asterisk but 
+	'TCP_Keepalive' default is disabled. Actives the tcp keep-alive at the socket layer. This does not require 
+	an event-loop and is lightweight. Useful for applications that use long-lived connections to Asterisk but 
 	do not run an event loop.
 	'Blocking' has a default of 1 (block on connecting). A value of 0 will cause us to queue our connection
 	and login for when an event loop is started. If set to non blocking we will always return a valid object.
@@ -79,22 +93,22 @@ Creates a new AMI object which takes the arguments as key-value pairs.
 	'on_connect_err', 'on_error', 'on_disconnect'
 	These three specify subroutines to call when errors occur. 'on_connect_err' is specifically for errors that
 	occur while connecting, as well as failed logins. If 'on_connect_err' or 'on_disconnect' it is not set, 
-	but 'on_error' is, 'on_error' wil be called. 'on_disconnect' is not reliable, as disconnects seem to get lumped
+	but 'on_error' is, 'on_error' will be called. 'on_disconnect' is not reliable, as disconnects seem to get lumped
 	under 'on_error' instead. When the subroutine specified for any of theses is called the first argument is a copy
 	of our AMI object, and the second is a string containing a message/reason. All three of these are 'fatal', when
 	they occur we destroy our buffers and our socket connections.
 
-	'on_timeout' is called when a keepalive has timed out, not when a normal action has. It is non-'fatal'.
+	'on_timeout' is called when a keep-alive has timed out, not when a normal action has. It is non-'fatal'.
 	The subroutine will be called with a copy of our AMI object and a message.
 	
-=head2 Warning - Mixing Eventloops and blocking actions
+=head2 Warning - Mixing Event-loops and blocking actions
 
 	If you are running an event loop and use blocking methods (anything that accepts it's timeout outside of 
 	the action hash e.g. get_response, check_response, action, connected) the outcome is unspecified. It may work,
 	it may lock everything up, the action may work but break something else. I have tested it and behavior seems
-	un-predictable at best and is very circumstantial.
+	unpredictable at best and is very circumstantial.
 
-	If you are running an eventloop use non-blocking callbacks! It is why they are there!
+	If you are running an event-loop use non-blocking callbacks! It is why they are there!
 
 	However if you do play with blocking methods inside of your loops let me know how it goes.
 
@@ -142,18 +156,18 @@ ActionID of the action, on success and will return undef in the event it is unab
 After sending an action you can then get its response in one of two methods.
 
 The method check_response() accepts an actionid and will return 1 if the action was considered successful, 0 if 
-it failed and undef if an error occured or on timeout.
+it failed and undef if an error occurred or on timeout.
 
 The method get_response() accepts an actionid and will return a Response object (really just a fancy hash) with the 
 contents of the Action Response as well as any associated Events it generated. It will return undef if an error 
-occured or on timeout.
+occurred or on timeout.
 
 All responses and events are buffered, therefor you can issue several send_action()s and then retrieve/check their 
-responses out of order without losing any information. Infact, if you are issuing many actions in series you can get 
+responses out of order without losing any information. In-fact, if you are issuing many actions in series you can get 
 much better performance sending them all first and then retrieving them later, rather than waiting for responses 
-immediatly after issuing an action.
+immediately after issuing an action.
 
-Alternativley you can also use simple_action() and action().
+Alternatively you can also use simple_action() and action().
 simple_action() combines send_action() and check_response(), and therefore returns 1 on success and 0 on failure,
 and undef on error or timeout.
 action() combines send_action() and get_response(), and therefore returns a Response object or undef.
@@ -212,14 +226,14 @@ action() combines send_action() and get_response(), and therefore returns a Resp
 In this example once the action 'Ping' finishes we will call somemethod() and pass it the a copy of our AMI object 
 and the Response Object for the action. If TIMEOUT is not specified it will use the default set. A value of 0 means 
 no timeout. When the timeout is reached somemethod() will be called and passed a reference to the our $astman and
-the un-completed Response Object, therefore somemethod() should check the state of the object. Checking the key {'GOOD'}
+the uncompleted Response Object, therefore somemethod() should check the state of the object. Checking the key {'GOOD'}
 is usually a good indication if the object is useable.
 
 Callback Caveats
 
 Callbacks only work if we are processing packets, therefore you must be running an event loop. Alternatively, we run 
 mini-event loops for our blocking calls (e.g. action(), get_action()), so in theory if you set callbacks and then
-issue a blocking call those callbacks should also get trigged. However this is an unsupported scenario.
+issue a blocking call those callbacks should also get triggered. However this is an unsupported scenario.
 
 Timeouts are done using timers, depending on how your event loop works it may be relative or absolute. Either way they are
 set as soon as you send the object. Therefore if you send an action with a timeout and then monkey around for a long time
@@ -250,9 +264,9 @@ This module handles ActionIDs internally and if you supply one in an action it w
 	$response->{'Response'}		Response to our packet (Success, Failed, Error, Pong, etc).
 		   {'ActionID'}		ActionID of this Response.
 		   {'Message'}		Message line of the response.
-		   {'EVENTS'}		Arrary reference containing Event Objects associated with this actionid.
-		   {'PARSED'}		Hash refernce of lines we could parse into key->value pairs.
-		   {'DATA'}		Array refernce of lines that we could not parse.
+		   {'EVENTS'}		Array reference containing Event Objects associated with this actionid.
+		   {'PARSED'}		Hash reference of lines we could parse into key->value pairs.
+		   {'DATA'}		Array reference of lines that we could not parse.
 		   {'CMD'}		Contains command output from 'Action: Command's. It is an array reference.
 		   {'COMPLETED'}	1 if completed, 0 if not (timeout)
 		   {'GOOD'}		1 if good, 0 if bad. Good means no errors and COMPLETED.
@@ -260,7 +274,7 @@ This module handles ActionIDs internally and if you supply one in an action it w
 
 =head3 Events
 
-	Events are turned into event objects, these are similiar to response objects, but their keys vary much more
+	Events are turned into event objects, these are similar to response objects, but their keys vary much more
 	depending on the specific event.
 
 	Some common contents are:
@@ -304,7 +318,7 @@ This module handles ActionIDs internally and if you supply one in an action it w
 	preferred event loop. We will use EV as our event loop in this example. I use subroutine references in this
 	example, but you could use anonymous subroutines if you want to.
 
-	#Use your prefered loop before our module so that AnyEvent will autodetect it
+	#Use your preferred loop before our module so that AnyEvent will auto-detect it
 	use EV;
 	use Asterisk::AMI:
 
@@ -316,7 +330,7 @@ This module handles ActionIDs internally and if you supply one in an action it w
 					Events		=>	'on',
 					Handlers	=>	{ default => \&eventhandler }
 				);
-	#Alternativly you can set Blocking => 0, and set an on_error sub to catch conneciton errors
+	#Alternatively you can set Blocking => 0, and set an on_error sub to catch connection errors
 	die "Unable to connect to asterisk" unless ($astman);
 
 	#Define the subroutines for events
@@ -337,7 +351,7 @@ This module handles ActionIDs internally and if you supply one in an action it w
 
 
 
-	Thats it, the EV loop will allow us to process input from asterisk. Once the action completes it will 
+	That's it, the EV loop will allow us to process input from asterisk. Once the action completes it will 
 	call the callback, and any events will be dispatched to eventhandler(). As you can see it is fairly
 	straight-forward. Most of the work will be in creating subroutines to be called for various events and 
 	actions that you plan to use.
@@ -346,7 +360,7 @@ This module handles ActionIDs internally and if you supply one in an action it w
 
 send_action ( ACTION )
 
-	Sends the action to asterisk. If no errors occured while sending it returns the ActionID for the action,
+	Sends the action to asterisk. If no errors occurred while sending it returns the ActionID for the action,
 	which is a positive integer above 0. If it encounters an error it will return undef.
 	
 check_response( [ ACTIONID ], [ TIMEOUT ] )
@@ -386,7 +400,7 @@ get_event ( [ TIMEOUT ] )
 
 amiver ()
 
-	Returns the version of the Asterisk Manager Interface we are connected to. Undef until a the connection is made
+	Returns the version of the Asterisk Manager Interface we are connected to. Undef until the connection is made
 	(important if you have Blocking => 0).
 	
 
@@ -441,10 +455,10 @@ use warnings;
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket;
-
+use Digest::MD5;
 
 #Duh
-use version; our $VERSION = qv(0.1.7);
+use version; our $VERSION = qv(0.1.8);
 
 #Keep track if we are logged in
 my $LOGGEDIN = 0;
@@ -469,7 +483,7 @@ my %ACTIONBUFFER;
 my %CALLBACKS;
 
 my $vertical;
-#Backwards compatability with 5.8, does not support \v, but on 5.10 \v is much faster than the below char class
+#Backwards compatibility with 5.8, does not support \v, but on 5.10 \v is much faster than the below char class
 {
 	no warnings;
 
@@ -537,14 +551,13 @@ my $myself;
 #Keep alive Anyevent::Timer
 my $keepalive;
 
-#Module wide condvar
-#my $process = AE::cv;
-
 #Defaults
 my $PEER = '127.0.0.1';
 my $PORT = '5038';
 my $USERNAME;
 my $SECRET;
+my $AUTHTYPE = 'plaintext';
+my $USESSL = 0;
 my $EVENTS = 'off';
 my $STOREEVENTS = 1;
 my $CALLBACK = 0;
@@ -554,8 +567,13 @@ my $TCPALIVE = 0;
 my $BUFFERSIZE = 30000;
 my $BLOCK = 1;
 my %EVENTHANDLERS;
+#Things to do on different errors.
 my %ON;
-my %DISCARD;
+#What to do with things for certain actions.
+my %EXPECTED;
+
+#Buffer for actions before login has occured.
+my %PRELOGIN;
 
 #Create a new object and return it;
 #If required options are missing, returns undef
@@ -601,7 +619,10 @@ sub _configure {
 		}
 	}
 
+	#Ugly any better way?
 	#Set values
+	$USESSL = $settings{'UseSSL'} if (defined $settings{'UseSSL'});
+	$PORT = 5039 if ($USESSL); #Change default port if using ssl
 	$PEER = $settings{'PeerAddr'} if (defined $settings{'PeerAddr'});
 	$PORT = $settings{'PeerPort'} if (defined $settings{'PeerPort'});
 	$USERNAME = $settings{'Username'} if (defined $settings{'Username'});
@@ -615,6 +636,7 @@ sub _configure {
 	$BUFFERSIZE = $settings{'BufferSize'} if (defined $settings{'BufferSize'});
 	%EVENTHANDLERS = %{$settings{'Handlers'}} if (defined $settings{'Handlers'});
 	$BLOCK = $settings{'Blocking'} if (defined $settings{'Blocking'});
+	$AUTHTYPE = $settings{'AuthType'} if (defined $settings{'AuthType'});
 
 	#On Connect
 	$ON{'connect'} = $settings{'on_connect'} if (defined $settings{'on_connect'});
@@ -735,20 +757,27 @@ sub _connect {
 
 	my $process = AE::cv;
 
-	$handle = new AnyEvent::Handle(
-		connect => [$PEER => $PORT],
-		keepalive => $TCPALIVE,
-		on_connect_err => sub { $self->_on_connect_err(1,$_[1]); },
-		on_error => sub { $self->_on_error($_[1],$_[2]) },
-		on_eof => sub { $self->_on_disconnect; },
-		on_connect => sub { $handle->push_read( line => \&_on_connect ); }
-	);
+	#Build a hash of our anyevent::handle options
+	my %hdl = (	connect => [$PEER => $PORT],
+			keepalive => $TCPALIVE,
+			on_connect_err => sub { $self->_on_connect_err(1,$_[1]); },
+			on_error => sub { $self->_on_error($_[1],$_[2]) },
+			on_eof => sub { $self->_on_disconnect; },
+			on_connect => sub { $handle->push_read( line => \&_on_connect ); });
 
+	#TLS stuff
+	$hdl{'tls'} = 'connect' if ($USESSL);
+
+	#Make connection/create handle
+	$handle = new AnyEvent::Handle(%hdl);
+
+	#Return login status if blocking
 	return $self->_login if ($BLOCK); 
 
 	#Queue our login
 	$self->_login;
 
+	#If we don't have a handle still fail
 	return 0 unless ($handle);
 
         return 1;
@@ -791,7 +820,7 @@ sub _sort_and_buffer {
 		#Snag our actionid
 		my $actionid = $packet->{'ActionID'};
 
-		return if ($DISCARD{$actionid});
+		return unless ($EXPECTED{$actionid});
 
 		if (exists $packet->{'Response'}) {
 			#No indication of future packets, mark as completed
@@ -836,7 +865,7 @@ sub _sort_and_buffer {
 				#cleanup
 				delete $ACTIONBUFFER{$actionid};
 				delete $CALLBACKS{$actionid};
-				delete $DISCARD{$actionid};
+				delete $EXPECTED{$actionid};
 				$callback->($myself, $action);
 			}
 		}
@@ -877,11 +906,16 @@ sub _gen_actionid {
 	return $idseq++;
 }
 
+#This is used to provide blocking behavior for calls
+#It installs callbacks for an action if it is not in the buffer and waits for the response before
+#returning it.
 sub _wait_response {
 	my ($id, $timeout) =  @_;
 
+	#Already got it?
 	unless ($ACTIONBUFFER{$id}->{'COMPLETED'}) {
 
+		#Install some handlers and use a CV to simulate blocking
 		my $process = AE::cv;
 
 		$CALLBACKS{$id}->{'cb'} = sub { $process->send($_[1]) };
@@ -892,7 +926,7 @@ sub _wait_response {
 					my $response = $ACTIONBUFFER{$id};
 					delete $ACTIONBUFFER{$id};
 					delete $CALLBACKS{$id};
-					$DISCARD{$id} = 1;
+					delete $EXPECTED{$id};
 					$process->send($response);
 				};
 
@@ -925,24 +959,14 @@ sub send_action {
 	delete $ACTIONBUFFER{$id};
 	delete $CALLBACKS{$id};
 
-	unless (defined $actionhash->{'TIMEOUT'}) {
-		$actionhash->{'TIMEOUT'} = $TIMEOUT;
-	}
+	#Set default timeout
+	$actionhash->{'TIMEOUT'} = $TIMEOUT unless (defined $actionhash->{'TIMEOUT'});
 
-	if (defined $actionhash->{'CALLBACK'}) {
-		$CALLBACKS{$id}->{'cb'} = $actionhash->{'CALLBACK'};
-		if ($actionhash->{'TIMEOUT'}) {
-			$CALLBACKS{$id}->{'timeout'} = sub {
-					my $response = $ACTIONBUFFER{$id};
-					my $callback = $CALLBACKS{$id}->{'cb'};
-					delete $ACTIONBUFFER{$id};
-					delete $CALLBACKS{$id};
-					$DISCARD{$id} = 1;
-					$callback->($self, $response);;
-				};
-			$CALLBACKS{$id}->{'timer'} = AE::timer $actionhash->{'TIMEOUT'}, 0, $CALLBACKS{$id}->{'timeout'};
-		}
-	}
+	#Assign Callback
+	$CALLBACKS{$id}->{'cb'} = $actionhash->{'CALLBACK'} if (defined $actionhash->{'CALLBACK'});
+
+	#Get a copy of our timeout
+	my $timeout = $actionhash->{'TIMEOUT'};
 
 	delete $actionhash->{'TIMEOUT'};
 	delete $actionhash->{'CALLBACK'};
@@ -968,10 +992,29 @@ sub send_action {
 	$action .= 'ActionID: ' . $id . $EOL . $EOR;	
 
 	#Send it!
-	$handle->push_write($action);
+	if ($LOGGEDIN || lc($actionhash->{'Action'}) =~ /login|challenge/) {
+		$handle->push_write($action);
+	} else {
+		$PRELOGIN{$id} = $action;
+	}
+
 	$ACTIONBUFFER{$id}->{'COMPLETED'} = 0;
 	$ACTIONBUFFER{$id}->{'GOOD'} = 0;
-	$DISCARD{$id} = 0;
+	$EXPECTED{$id} = 1;
+
+	#Start timer for timeouts
+	if ($timeout && defined $CALLBACKS{$id}) {
+		$CALLBACKS{$id}->{'timeout'} = sub {
+				my $response = $ACTIONBUFFER{$id};
+				my $callback = $CALLBACKS{$id}->{'cb'};
+				delete $ACTIONBUFFER{$id};
+				delete $CALLBACKS{$id};
+				delete $EXPECTED{$id};
+				delete $PRELOGIN{$id};
+				$callback->($self, $response);;
+			};
+		$CALLBACKS{$id}->{'timer'} = AE::timer $timeout, 0, $CALLBACKS{$id}->{'timeout'};
+	}
 
 	return $id;
 }
@@ -1052,21 +1095,79 @@ sub simple_action {
 sub _login {
 	my $self = shift;
 
-	my %action = ( 	Action => 'login',
-			Username =>  $USERNAME,
-			Secret => $SECRET,
-			Events => $EVENTS
-	);
+	#Auth challenge
+	my %challenge;
+	
+	#Build login action
+	my %action = (	Action => 'login',
+			Username => $USERNAME,
+			Events => $EVENTS );
 
+	#Actions to take for different authtypes
+	if (lc($AUTHTYPE) eq 'md5') {
+		#Do a challenge
+		%challenge = (	Action => 'Challenge',
+				AuthType => $AUTHTYPE);
+	} else {
+		$action{'Secret'} = $SECRET;
+	}
+
+	#Blocking connect
 	if ($BLOCK) {
-		
-		my $resp = $self->action(\%action);		
+		my $resp;
 
+		my $timeout = 5 unless ($TIMEOUT);
+
+		#If a challenge exists do handle it first before the login
+		if (%challenge) {
+			#Get challenge response
+			my $chresp = $self->action(\%challenge);
+
+			if ($chresp->{'GOOD'}) {
+				#Build up our login from the challenge
+				my $md5 = new Digest::MD5;
+
+				$md5->add($chresp->{'PARSED'}->{'Challenge'});
+				$md5->add($SECRET);
+
+				$md5 = $md5->hexdigest;
+
+				$action{'Key'} = $md5;
+				$action{'AuthType'} = $AUTHTYPE;
+
+				#Login
+				$resp = $self->action(\%action,$timeout);
+						
+			} else {
+				#Challenge Failed
+				if ($chresp->{'COMPLETED'}) {
+					warn "$AUTHTYPE challenge failed";
+				} else {
+					warn "Timed out waiting for challenge";
+				}
+			}
+		} else {
+			#Plaintext login
+			$resp = $self->action(\%action,$timeout);
+		}
+
+		
 		if ($resp->{'GOOD'}){
+			#Login successful
 			$LOGGEDIN = 1;
+			#Run on_connect stuff
 			$ON{'connect'}->($self) if (defined $ON{'connect'});
+
+			#Flush pre-login buffer			
+			foreach (values %PRELOGIN) {
+				$handle->push_write($_);
+			}
+
+			undef %PRELOGIN;
+
 			return 1;
 		} else {
+			#Login Failed
 			$LOGGEDIN = 0;
 			if ($resp->{'COMPLETED'}) {
 				warn "Authentication Failed";
@@ -1074,12 +1175,23 @@ sub _login {
 				warn "Timed out waiting for login";
 			}
 		}
+	#Non-blocking connect
 	} else {
+
+		#Callback for login action
 		$action{'CALLBACK'} = sub {
-					if ($_[1]->{'GOOD'}) {	
+					if ($_[1]->{'GOOD'}) {
+						#Login was good
 						$LOGGEDIN = 1;
+						#Flush pre-login buffer			
+						foreach (values %PRELOGIN) {
+							$handle->push_write($_);
+						}
+						undef %PRELOGIN;
+
 						$ON{'connect'}->($self) if (defined $ON{'connect'});
 					} else {
+						#Login failed
 						my $message;
 
 						if ($_[1]->{'COMPLETED'}) {
@@ -1094,7 +1206,39 @@ sub _login {
 
 		$action{'TIMEOUT'} = 5 unless ($TIMEOUT);
 
-		$self->send_action(\%action);
+		#Do a md5 challenge
+		if (%challenge) {
+			#Create callbacks for the challenge
+			$challenge{'TIMEOUT'} = 5 unless ($TIMEOUT);
+			$challenge{'CALLBACK'} = sub {
+				if ($_[1]->{'GOOD'}) {
+					my $md5 = new Digest::MD5;
+
+					$md5->add($_[1]->{'PARSED'}->{'Challenge'});
+					$md5->add($SECRET);
+
+					$md5 = $md5->hexdigest;
+
+					$action{'Key'} = $md5;
+					$action{'AuthType'} = $AUTHTYPE;
+
+					$self->send_action(\%action);
+						
+				} else {
+					if ($_[1]->{'COMPLETED'}) {
+						warn "$AUTHTYPE challenge failed";
+					} else {
+						warn "Timed out waiting for challenge";
+					}
+				}
+			};
+			#Send challenge
+			$self->send_action(\%challenge);
+
+		} else { 
+			#Plaintext login
+			$self->send_action(\%action);
+		}
 
 		return 1;
 	}
@@ -1200,7 +1344,7 @@ sub _clear_cbs {
 		my $callback = $CALLBACKS{$id}->{'cb'};
 		delete $ACTIONBUFFER{$id};
 		delete $CALLBACKS{$id};
-		$DISCARD{$id} = 1;
+		$EXPECTED{$id} = 1;
 		$callback->($myself, $response);
 	}
 }
